import ble_store from "../store/bluetooth";

export default {
  /**
   * 获取蓝牙设备服务
   * @param deviceId 脑机mac
   */
  getBLEDeviceServices(deviceId) {
    const that = this;
    wx.getBLEDeviceServices({
      deviceId,
      success: (res) => {
        console.log("获取蓝牙设备服务service:\n", JSON.stringify(res.services));
        for (let i = 0; i < res.services.length; i++) {
          console.log("第" + (i + 1) + "个UUID:" + res.services[i].uuid + "\n");
          if (res.services[i].uuid.indexOf('6E') !== -1 || res.services[i].uuid.indexOf('0000FFF0') !== -1) {
            // 获取蓝牙设备某个服务中所有特征值
            that.getBLEDeviceCharacteristics(deviceId, res.services[i].uuid);
            ble_store.setters.setServiceId(res.services[i].uuid);
            console.log("脑机deviceId(mac)", deviceId, "notifyServicesId:" + res.services[i].uuid);
            return;
          }
        }
      },
      fail(res) {
        let deviceId = wx.getStorageSync('deviceId');
        //断开蓝牙连接
        wx.closeBLEConnection({
          deviceId: deviceId
        });
      },
    });
  },

  /**
   * 获取蓝牙设备某个服务中所有特征值
   */
  getBLEDeviceCharacteristics(deviceId, serviceId) {
    wx.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
      success: (res) => {
        console.log("获取服务", serviceId, "的特征值:\n", JSON.stringify(res));
        for (let i = 0; i < res.characteristics.length; i++) {
          let item = res.characteristics[i]
          if (item.properties.read) {
            console.log("第" + (i + 1) + ",该特征值可读：" + item.uuid);
          }
          if (item.properties.write) {
            console.log("第" + (i + 1) + ",该特征值可写：" + item.uuid);
            if(item.uuid.indexOf("0002") !== -1){
              ble_store.setters.setCharacteristicId(item.uuid);
            }
          }
          if (item.properties.notify || item.properties.indicate) {
            console.log("第" + (i + 1) + ",该特征值可监听：" + item.uuid);
            // 必须先启用 wx.notifyBLECharacteristicValueChange 才能监听到设备 onBLECharacteristicValueChange 的事件
            wx.notifyBLECharacteristicValueChange({
              deviceId,
              serviceId,
              characteristicId: item.uuid,
              state: true,
            });
          }
        }
      },
      fail(res) {
        let deviceId = wx.getStorageSync('deviceId');
        //断开蓝牙连接
        wx.closeBLEConnection({
          deviceId: deviceId
        });
      },
    });
  },

  /**
   * 脑机连接教具
   */
  sendToyConnection(toyItem) {
    if(toyItem && toyItem["hex"]) {
      let $hex = $this.toy_item["hex"].substr($this.toy_item["hex"].length - 2, 2);
      if ($hex === "80") {
        wx.setStorageSync("report_mode", 2)
      } else {
        wx.setStorageSync("report_mode", 1)
      }
      console.log("连接教具:", `03 00 ${$hex} 00 0A`, JSON.stringify($this.toy_item));
      // 连接教具: 03 00 ${$hex} 00 0a
      $this.$bluetooth.sendConnectOneToMore($hex);
    }
  },
  /**
   *  发送一对多连接
   */
  sendConnectOneToMore(id) {
    this.WriteBufferInBle(`03 00 ${id} 00 0A`);
  },
  /**
   * todo 写入buffer
   * @param $hex
   * @param $buffer_len
   * @constructor
   */
  WriteBufferInBle($hex, $buffer_len = 8) {
    let deviceId = ble_store.getters.getDeviceId();
    let serviceId = ble_store.getters.getServiceId();
    let characteristicId = ble_store.getters.getCharacteristicId()
    if (deviceId && serviceId && characteristicId) {
      let that = this;
      let $hex_header = "aa cc ";
      let $hex_sum = 0;
      let $hex_ary = $hex.split(" ");
      $hex_ary.forEach(($val, $index) => {
        $hex_sum += parseInt($val, 16);
      })
      let $checksum = ($hex_sum ^ parseInt("ffffffff", 16)) & parseInt("ff", 16);
      $hex = $hex_header + $hex + " " + ("00" + $checksum.toString(16)).substr(-2, 2);
      let buffer = new ArrayBuffer($buffer_len);
      let dataView = new DataView(buffer);
      $hex_ary = $hex.split(" ");
      $hex_ary.forEach(($val, $index) => {
        dataView.setUint8($index, parseInt($val, 16))
      })
      wx.writeBLECharacteristicValue({
        deviceId: deviceId,
        serviceId: serviceId,
        characteristicId: characteristicId,
        value: buffer,
        success: function (res) {
          console.log($hex + "写入成功，时间：" + that.getNowTime())
        },
        fail: function (err) {
          console.log($hex + "写入失败", err);
        },
      });
    }
  },
};
